__author__ = 'tarzan'
import re
import inspect
import itertools
from collections import OrderedDict
from pyramid.decorator import reify

_managers_factory = []


def _name_to_underscore(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


def _name_to_words(name):
    name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    words = re.split('[_\s]+', name)

    def capitalize(s):
        return s[0].upper() + s[1:]

    return ' '.join([capitalize(w) for w in words])


def create_backend_manager(model):
    """
    create backend manager for a model
    :rtype Manager
    """
    for mf in _managers_factory:
        mgr = mf(model)
        if mgr:
            return mgr

    raise NotImplementedError('Can not find backend manager for model "%s"' % model.__name__)


def register_manager_factory(factory):
    global _managers_factory
    _managers_factory.append(factory)


def get_manager(model):
    """
    get backend manager for a model
    :rtype Manager
    """
    try:
        return model.__backend_manager__
    except AttributeError:
        manager = create_backend_manager(model)
        model.__backend_manager__ = manager
        return manager


class AttrDisplayConf(object):
    def __init__(self, attr_name, *args):
        self.attr_name = attr_name

        def first_instance(types, default):
            for arg in args:
                if isinstance(arg, types):
                    return arg
            return default

        self.label = first_instance(basestring, _name_to_words(attr_name))
        self.limit = first_instance((int, long), 5)

    def values(self, obj):
        vals = self.value(obj)
        try:
            vals = itertools.islice(vals, self.limit)
        except TypeError:
            vals = [vals, ]
        return vals

    def value(self, obj):
        return obj.__getattribute__(self.attr_name)


class Manager(object):

    __attr_display_conf_class__ = AttrDisplayConf

    def __init__(self, model):
        """
        :type model: class
        """
        assert inspect.isclass(model)
        self.Model = model
        self.actions = OrderedDict()

    @property
    def column_names(self):
        return []

    @reify
    def default_actions(self):
        return OrderedDict([
            ('list', {
                'route_name': 'admin_site',
                'context': self.ModelResource,
                'name': '',
                'attr': 'action_list',
                'renderer': 'pyramid_backend:templates/list.mak',
                'permission': 'list',
                '_icon': 'list',
                '_label': self.display_name + u' list',
            }),
            ('create', {
                'route_name': 'admin_site',
                'context': self.ModelResource,
                'name': 'create',
                'attr': 'action_create',
                'renderer': 'pyramid_backend:templates/create.mak',
                'permission': 'create',
                '_icon': 'plus',
                '_label': u'Create new ' + self.display_name,
            }),
            ('detail', {
                'route_name': 'admin_site',
                'context': self.ObjectResource,
                'name': '',
                'attr': 'action_detail',
                'renderer': 'pyramid_backend:templates/detail.mak',
                'permission': 'detail',
                '_icon': 'eye-open',
                '_label': u'View %s detail',
            }),
            ('update', {
                'route_name': 'admin_site',
                'context': self.ObjectResource,
                'name': 'update',
                'attr': 'action_update',
                'renderer': 'pyramid_backend:templates/update.mak',
                'permission': 'update',
                '_icon': 'pencil',
                '_label': u'Update %s',
            }),
            ('delete', {
                'route_name': 'admin_site',
                'context': self.ObjectResource,
                'name': 'delete',
                'attr': 'action_delete',
                'renderer': 'pyramid_backend:templates/update.mak',
                'permission': 'delete',
                '_icon': 'remove',
                '_label': u'Delete %s',
                '_onclick': u"return confirm('Do you want to delete %s?');",
            }),
        ]
        )

    _configurable_properties = [
        'slug',
        '__acl__',
        'display_name',
        'schema_cls',
        'id_attr',
        'list__columns_to_display',
        'detail__columns_to_display',
        'list__items_per_page',
        'detail__relations_to_display',
    ]

    _display_config = [
        'list__columns_to_display',
        'detail__columns_to_display',
        'detail__relations_to_display',
    ]

    def _make_attr_display_config(self, conf):
        if not isinstance(conf, (list, tuple)):
            conf = [conf, ]
        return self.__attr_display_conf_class__(*conf)

    def __getattribute__(self, name):
        if name in Manager._configurable_properties:
            try:
                val = getattr(self.Model, '__backend_' + name + '__')
            except AttributeError:
                val = getattr(self, '__default_' + name + '__')
            if name in Manager._display_config:
                val = [self._make_attr_display_config(v) for v in val]
            if name == 'id_attr':
                if not isinstance(val, (list, tuple)):
                    val = (val, )
            setattr(self, name, val)
            return val
        return super(Manager, self).__getattribute__(name)

    __default_schema_cls__ = None
    __default_id_attr__ = 'id'
    __default_list__items_per_page__ = 50

    @property
    def __default_slug__(self):
        return _name_to_underscore(self.Model.__name__)

    @property
    def __default_display_name__(self):
        return _name_to_words(self.Model.__name__)

    @property
    def __default_list__columns_to_display__(self):
        columns = dict(zip(dir(self.Model), dir(self.Model)))
        columns[self.id_attr] = '#'
        return columns

    @property
    def __default_detail__columns_to_display__(self):
        columns = dict(zip(dir(self.Model), dir(self.Model)))
        columns[self.id_attr] = '#'
        return columns

    @property
    def __default_detail__relations_to_display__(self):
        return {}

    @reify
    def ModelResource(self):
        from pyramid_backend import resources

        return resources.model_resource_class(self.Model)

    @reify
    def ObjectResource(self):
        from pyramid_backend import resources

        return resources.object_resource_class(self.Model)

    def get_id_filters(self, id_value):
        id_part_count = len(self.id_attr)
        vals = str(id_value).split('-', id_part_count)
        return dict(zip(self.id_attr, vals))

    def normalize_action(self, action_conf):
        assert bool(action_conf), 'Can not be empty configuration'
        if not isinstance(action_conf, (list, tuple,)):
            assert action_conf in self.default_actions, 'Unknown action named "%s"' % action_conf
            action_conf = (action_conf, self.default_actions[action_conf])
        return action_conf

    def add_action(self, action_conf):
        action_name, action_conf = self.normalize_action(action_conf)
        self.actions[action_name] = action_conf
        return action_name, action_conf

    def object_id(self, obj):
        return u'-'.join([unicode(getattr(obj, attr_name)) for attr_name in self.id_attr])

    def create(self, data):
        raise NotImplementedError()

    def update(self, obj, data):
        raise NotImplementedError()

    def fetch_objects(self, filters, page=1):
        raise NotImplementedError()

    def count_objects(self, filters):
        raise NotImplementedError()

    def find_object(self, id_value):
        id_filters = self.get_id_filters(id_value)
        objs = self.fetch_objects(id_filters)
        return objs[0] if objs else None



__author__ = 'tarzan'

import re
import importlib
from backend_manager import register_manager_factory

ADMIN_SITE_PATH = None

def includeme(config):
    """
    :type config: pyramid.config.Configurator
    """
    global ADMIN_SITE_PATH
    settings = config.get_settings()
    ADMIN_SITE_PATH = settings['pyramid_backend.admin_site']
    ADMIN_SITE_PATH = ADMIN_SITE_PATH.strip('/') + '/'

    from . import resources as _rsr
    from . import model as model_helper
    from . import views

    route_pattern = ADMIN_SITE_PATH + '*traverse'
    config.add_route('admin_site', route_pattern, factory=_rsr.AdminSite)
    config.add_directive('add_model_view', views.add_model_view)
    config.add_directive('add_model_action', views.add_model_action)
    config.add_directive('add_object_action', views.add_object_action)

    factories_path = settings.get('pyramid_backend.manager_factories', """
    pyramid_backend.backend_manager.sqlalchemy.factory
    pyramid_backend.backend_manager.cqlengine.factory
    """)
    factories_path = filter(bool, re.split('\s+', factories_path))
    for factory in factories_path:
        if not callable(factory):
            _module, _var = factory.rsplit('.', 1)
            _module = importlib.import_module(_module, package=None)
            factory = getattr(_module, _var)

        register_manager_factory(factory(config))

    model_paths = settings.get('pyramid_backend.models', '')
    model_paths = filter(bool, re.split('\s+', model_paths))
    for path in model_paths:
        if '#' in path:
            path, actions = path.split('#', 1)
            actions = re.split(r'[\s,]+', actions.strip())
        else:
            actions = None
        _module, _var = path.rsplit('.', 1)
        _module = importlib.import_module(_module, package=None)
        model = getattr(_module, _var)
        config.add_model_view(model, actions=actions)

    config.scan(__name__)


__author__ = 'tarzan'

import urllib
import inspect
import deform
from webhelpers.paginate import Page
from pyramid.httpexceptions import HTTPFound
from . import cell_datatype
from .. import resources as _rsr
import json
import colander
from pyramid.traversal import lineage


def _none_to_colander_null(data):
    return {k: v if v is not None else colander.null for k, v in data.items()}


def _colander_null_to_none(data):
    return {k: v if v is not colander.null else None for k, v in data.items()}


class ModelView(object):
    def __init__(self, context, request):
        """
        :type context: pyramid_backend.resources.ModelResource
        :type request: pyramid.request.Request
        """
        self.context = context
        self.request = request

    @property
    def model(self):
        return self.context.model

    @property
    def is_current_context_object(self):
        """
        :rtype : bool
        """
        return isinstance(self.context, _rsr.ObjectResource)

    @property
    def backend_mgr(self):
        """
        :rtype : pyramid_backend.backend_manager.Manager
        """
        return self.model.__backend_manager__

    @property
    def model_schema_cls(self):
        assert inspect.isclass(self.backend_mgr.schema_cls), \
            '%s.__backend_schema_cls__ (%s) is not a class' % \
            (self.model.__name__, self.backend_mgr.schema_cls)
        return self.backend_mgr.schema_cls

    def cell_datatype(self, val):
        return cell_datatype(val)

    @property
    def toolbar_actions(self):
        actions = self.model_actions
        if isinstance(self.context, self.backend_mgr.ObjectResource):
            actions += self.object_actions(self.context.object)
        return actions

    @property
    def model_actions(self):
        configured_actions = self.backend_mgr.actions
        actions = []
        for ca_name, ca in configured_actions.items():
            cxt = ca['context']
            if issubclass(cxt, _rsr.ModelResource):
                _label = ca['_label'] if '_label' in ca else \
                    (self.backend_mgr.display_name + '@' + ca_name)
                actions.append({
                    'url': _rsr.model_url(self.request, self.model, ca.get('name', None)),
                    'label': _label,
                    'icon': ca['_icon'] if '_icon' in ca else None,
                })
        return actions

    def object_actions(self, obj):
        actions = []
        for ca_name, ca in self.backend_mgr.actions.items():
            cxt = ca['context']
            if issubclass(cxt, _rsr.ObjectResource):
                _label = ca['_label'] if '_label' in ca else ('%s@' + ca_name)
                _label = _label % obj
                _onclick = json.dumps(ca['_onclick'] % obj).strip(
                    '"\'') if '_onclick' in ca else None
                actions.append({
                    'url': _rsr.object_url(self.request, obj, ca.get('name', None)),
                    'label': _label,
                    'icon': ca['_icon'] if '_icon' in ca else None,
                    'onclick': _onclick,
                })
        return actions

    @property
    def breadcrumbs(self):
        cxt = self.context
        cxts = list(reversed(list(lineage(cxt))))
        r = self.request
        if not r.view_name:
            if self.is_current_context_object:
                cmd_name = 'detail'
            else:
                cmd_name = 'list'
            cxts = cxts[:-1]
        else:
            cmd_name = r.view_name

        if cmd_name in self.backend_mgr.actions:
            ca = self.backend_mgr.actions[cmd_name]
            if self.is_current_context_object:
                _label = ca['_label'] if '_label' in ca else ('%s@' + cmd_name)
                _label = _label % cxt.object
            else:
                _label = ca['_label'] if '_label' in ca else \
                    (self.backend_mgr.display_name + '@' + cmd_name)
        else:
            _label = '@' + cmd_name

        return [{
            'url': r.resource_url(c),
            'label': u'%s' % c,
        } for c in cxts] + [_label, ]

    def list_page_url(self, page, partial=False):
        params = self.request.GET.copy()
        params["_page"] = page
        if partial:
            params["partial"] = "1"
        qs = urllib.urlencode(params, True)
        return "%s?%s" % (self.request.path, qs)

    def action_list(self):
        cur_page = int(self.request.params.get("_page", 1))
        filters = {k: v for k, v in self.request.GET.items() if not k.startswith('_')}
        objects = self.backend_mgr.fetch_objects(page=cur_page, filters=filters)
        objects = list(objects)
        objects_count = self.backend_mgr.count_objects(filters)
        # objects_count = objects_count * 20
        page = Page(objects, page=cur_page,
                    item_count=objects_count,
                    items_per_page=self.backend_mgr.list__items_per_page,
                    url=self.list_page_url,
                    presliced_list=True,
        )
        return {
            'view': self,
            'backend_mgr': self.backend_mgr,
            'page': page,
        }

    def action_create(self):
        schema = self.model_schema_cls().bind()
        form = deform.Form(schema,
                           buttons=(deform.Button(title='Create'),
                                    deform.Button(title='Cancel', type='reset', name='cancel')))

        if 'submit' in self.request.POST:
            try:
                data = form.validate(self.request.POST.items())
                data = _colander_null_to_none(data)
                obj = self.backend_mgr.create(data)
                self.request.session.flash(u'"%s" was created successful.' % obj, queue='pbackend')
                return HTTPFound(_rsr.object_url(self.request, obj))
            except deform.ValidationFailure as e:
                pass

        return {
            'view': self,
            "form": form,
        }

    def action_update(self):
        obj = self.context.object
        schema = self.model_schema_cls().bind(obj=obj)
        """:type schema: colander.Schema"""
        appstruct = _none_to_colander_null({k: obj.__getattribute__(k)
                                            for k in self.backend_mgr.column_names})
        form = deform.Form(schema,
                           appstruct=appstruct,
                           buttons=(deform.Button(title='Update'),
                                    deform.Button(title='Cancel', type='reset', name='cancel')))

        if 'submit' in self.request.POST:
            try:
                data = form.validate(self.request.POST.items())
                data = _colander_null_to_none(data)
                obj = self.backend_mgr.update(obj, data)
                self.request.session.flash(u'"%s" was updated successful.' % obj, queue='pbackend')
                return HTTPFound(_rsr.object_url(self.request, obj))
            except deform.ValidationFailure as e:
                pass

        return {
            'view': self,
            'obj': obj,
            "form": form,
        }

    def action_detail(self):
        return {
            'view': self,
            'obj': self.context.object,
            'backend_mgr': self.backend_mgr,
        }

    def action_delete(self):
        obj = self.context.object
        self.backend_mgr.delete(obj)
        self.request.session.flash(u'%s#%s was history!' % (self.backend_mgr.display_name, obj))
        return HTTPFound(_rsr.model_url(self.request, self.model))



__author__ = 'tarzan'

import urllib
from . import model as model_helper
import pyramid_backend as pb


def model_url(request, model, action=None, query=None):
    """
    Get url for an model with its action
    :type request: pyramid.request.Request
    :type model: object
    :type action: string
    :type query: string or dict
    """
    model = model_helper.get_registered_model(model)

    url = pb.ADMIN_SITE_PATH + model.__backend_manager__.slug
    if action:
        url += '/' + action
    if query and isinstance(query, dict):
        query = urllib.urlencode(query)
    if query:
        url += '?' + query
    return request.relative_url(url, True)


def object_url(request, obj, action=None, query=None):
    if not obj:
        return ''
    if isinstance(obj, type):
        model = obj
    else:
        model = obj.__class__
    obj_id = model.__backend_manager__.object_id(obj)
    url = pb.ADMIN_SITE_PATH + model.__backend_manager__.slug + '/' + str(obj_id)
    if action:
        url += '/' + action
    if query and isinstance(query, dict):
        query = urllib.urlencode(query)
    if query:
        url += '?' + query
    return request.relative_url(url, True)


class ModelResource(object):
    model = None

    def __init__(self, parent, name):
        """
        :type parent: AdminSite
        """
        self.__parent__ = parent
        self.__name__ = name
        self.request = parent.request

    @property
    def backend_mgr(self):
        """
        :rtype : pyramid_backend.backend_manager.Manager
        """
        return self.model.__backend_manager__

    def __getitem__(self, id_value):
        obj = self.backend_mgr.find_object(id_value)
        if obj:
            return self.backend_mgr.ObjectResource(self, id_value, obj)
        raise KeyError('%s#%s not found' % (self.backend_mgr.display_name, id_value))

    def __resource_url__(self, *args, **kwargs):
        return model_url(self.request, self.model)

    def __unicode__(self):
        return unicode(self.model.__name__)

    def __str__(self):
        return self.__unicode__().encode('utf-8')


class ObjectResource(object):
    url = '#'

    def __init__(self, parent, name, object):
        """
        :type parent: ModelResource
        """
        self.__parent__ = parent
        self.__name__ = name
        self.model = parent.model
        self.request = parent.request
        self.object = object

    def __resource_url__(self, *args, **kwargs):
        return object_url(self.request, self.object)

    def __unicode__(self):
        return u'%s' % self.object

    def __str__(self):
        return self.__unicode__().encode('utf-8')


_AUTO_CLASSES = {}


def model_resource_class(model):
    cls_name = model.__name__ + '_ModelResource'
    if cls_name not in _AUTO_CLASSES:
        _AUTO_CLASSES[cls_name] = type(cls_name, (ModelResource,), {
            'model': model,
        })

    try:
        _AUTO_CLASSES[cls_name].__acl__ = model.__backend_manager__.__acl__
    except AttributeError:
        pass

    return _AUTO_CLASSES[cls_name]


def object_resource_class(model):
    cls_name = model.__name__ + '_ObjectResource'
    if cls_name not in _AUTO_CLASSES:
        _AUTO_CLASSES[cls_name] = type(cls_name, (ObjectResource,), {
            'model': model,
        })

    return _AUTO_CLASSES[cls_name]


class AdminSite(object):
    __acl__ = [
    ]
    model_mappings = {

    }

    __parent__ = None
    __name__ = ''
    # @property
    # def __name__(self):
    #     return pb.ADMIN_SITE_PATH.strip('/')

    def __init__(self, request):
        """
        :type request: pyramid.request.Request
        """
        self.request = request

    def __getitem__(self, item):
        try:
            model = AdminSite.model_mappings[item]
        except KeyError as e:
            for model in model_helper.get_registered_models():
                if model.__backend_manager__.slug == item:
                    AdminSite.model_mappings[item] = model
                    break
            else:
                raise e
        return model.__backend_manager__.ModelResource(self, item)

    def __resource_url__(self, *args, **kwargs):
        return self.request.application_url + '/' + pb.ADMIN_SITE_PATH.strip('/') + '/'

    def __unicode__(self):
        return u'Backend'

    def __str__(self):
        return 'Backend'



from __future__ import absolute_import
__author__ = 'tarzan'

import inspect
from collections import OrderedDict
from pyramid.decorator import reify
import importlib
from . import Manager, _name_to_words, AttrDisplayConf as _BaseAttrDisplayConf
from cqlengine.models import (
    Model as CQLEngineModel,
)
from cqlengine.exceptions import  ValidationError
from cqlengine.query import ModelQuerySet


def factory(config):
    """
    :type config: pyramid.config.Configurator
    """
    def _create_manager(model):
        if inspect.isclass(model) and issubclass(model, CQLEngineModel):
            return CQLEngineManager(model)
        return None

    return _create_manager


class AttrDisplayConf(_BaseAttrDisplayConf):
    def values(self, obj):
        vals = self.value(obj)
        if isinstance(vals, ModelQuerySet):
            vals = list(vals)
        if isinstance(vals, (list, tuple)):
            return vals[:self.limit]
        return [vals, ]


class CQLEngineManager(Manager):

    __attr_display_conf_class__ = AttrDisplayConf

    @reify
    def queryset(self):
        """
        Get query set for current model
        :rtype cqlengine.query.ModelQuerySet
        """
        return self.Model.objects

    @reify
    def column_names(self):
        return [c_name for c_name in self.Model._columns]

    def column(self, col_name):
        return getattr(self.Model, col_name)

    @property
    def __default_list__columns_to_display__(self):
        columns = OrderedDict(zip(self.column_names, [_name_to_words(n) for n in self.column_names]))
        columns[self.id_attr] = '#'
        return columns

    @property
    def __default_detail__columns_to_display__(self):
        columns = OrderedDict(zip(self.column_names, [_name_to_words(n) for n in self.column_names]))
        columns[self.id_attr] = '#'
        return columns

    @property
    def __default_detail__relations_to_display__(self):
        return []

    def create(self, data):
        obj = self.Model.create(**data)
        return obj

    def update(self, obj, data):
        obj.update(**data)
        return obj

    def delete(self, obj):
        obj.delete()

    def fetch_objects(self, filters, page=1):
        criteria = []
        for name, value in filters.items():
            if name in self.column_names:
                criteria.append(self.column(name) == value)
        limit = self.list__items_per_page
        offset = (page-1)*limit
        objs = self.queryset.filter(*criteria).limit(offset + limit)
        return list(objs)[-limit:]

    def count_objects(self, filters):
        return 10000
        criteria = []
        for name, value in filters.items():
            if name in self.column_names:
                criteria.append(self.column(name) == value)
        return self.queryset.filter(*criteria).count()



__author__ = 'tarzan'

import inspect
from .backend_manager import create_backend_manager

_registered_models = [

]

def get_model_url(request, model, action=None):
    model = get_registered_model(model)
    traverse = [model.__backend_manager__.slug,]
    if action:
        traverse.append(action)
    return request.route_url('admin_site',
                             traverse=traverse)

def get_registered_models():
    return _registered_models

def get_registered_model(model):
    for m in _registered_models:
        if m is model:
            return m
    raise NotImplementedError('%s model has not been implemented' % model.__name__)

def is_registered_model(obj):
    if not inspect.isclass(obj):
        obj = type(obj)
        if not inspect.isclass(obj):
            return False
    return obj in _registered_models

def register_model(model):
    global _registered_models
    if model in _registered_models:
        return
    model.__backend_manager__ = create_backend_manager(model)
    _registered_models.append(model)


from __future__ import absolute_import
__author__ = 'tarzan'

from collections import OrderedDict
from pyramid.decorator import reify
import importlib
from . import Manager, _name_to_words


def factory(config):
    return lambda m: None

try:
    from sqlalchemy import func, Table
    _sqlalchemy_is_available = True
except ImportError:  # sqlalchemy is not available
    _sqlalchemy_is_available = False
    func = object()

    class Table(object):
        pass


if _sqlalchemy_is_available:
    DBSession = None
    """:type : sqlalchemy.orm.session.Session"""

    def factory(config):
        """
        :type config: pyramid.config.Configurator
        """
        global DBSession
        settings = config.get_settings()
        dbsession_path = settings.get('pyramid_backend.sqlalchemy.dbsession')
        if not dbsession_path:
            return lambda m: None

        _module, _var = dbsession_path.rsplit('.', 1)
        _module = importlib.import_module(_module, package=None)
        DBSession = getattr(_module, _var)

        def _create_manager(model):
            # verify model is an ORM class of SQLAlchemy or not
            # if not, just return None
            try:
                table = model.__table__
            except AttributeError:
                return None
            if not isinstance(table, Table):
                return None
            return SQLAlchemyManager(model)

        return _create_manager

    class SQLAlchemyManager(Manager):

        @reify
        def column_names(self):
            return [c.name for c in self.Model.__table__.columns]

        def column(self, col_name):
            return getattr(self.Model, col_name)

        @property
        def _foreignkey_names(self):
            return [rel.key for rel in self.Model.__mapper__.relationships]

        @property
        def __default_list__columns_to_display__(self):
            columns = OrderedDict(zip(
                self.column_names,
                [_name_to_words(n) for n in self.column_names]
            ))
            columns[self.id_attr] = '#'
            return columns

        @property
        def __default_detail__columns_to_display__(self):
            columns = OrderedDict(zip(
                self.column_names,
                [_name_to_words(n) for n in self.column_names]
            ))
            columns[self.id_attr] = '#'
            return columns

        @property
        def __default_detail__relations_to_display__(self):
            return OrderedDict(zip(
                self._foreignkey_names,
                [_name_to_words(n) for n in self._foreignkey_names]
            ))

        def create(self, data):
            obj = self.Model()
            for k, v in data.items():
                setattr(obj, k, v)
            DBSession.add(obj)
            DBSession.flush()
            return obj

        def update(self, obj, data):
            for k, v in data.items():
                setattr(obj, k, v)
            DBSession.merge(obj)
            DBSession.flush()
            return obj

        def delete(self, obj):
            DBSession.delete(obj)
            DBSession.flush()

        def fetch_objects(self, filters, page=1):
            query = DBSession.query(self.Model)
            for name, value in filters.items():
                if name in self.column_names:
                    query = query.filter(self.column(name).like("%%%s%%" % value))
            limit = self.list__items_per_page
            offset = (page-1)*limit
            id_col = self.column(self.id_attr)
            query = query.order_by(id_col.desc())
            query = query.offset(offset).limit(limit)
            return query

        def count_objects(self, filters):
            id_col = self.column(self.id_attr)
            query = DBSession.query(func.count(id_col))
            for name, value in filters.items():
                if name in self.column_names:
                    query = query.filter(self.column(name).like("%%%s%%" % value))
            return query.scalar()

        # def find_object(self, id_value):
        #     return DBSession.query(self.Model).filter(self.column(self.id_attr) == id_value).first()


__author__ = 'tarzan'

from pyramid.view import view_config
from datetime import datetime
import inspect
from pyramid_backend import resources as _rsr
from pyramid_backend import model as model_helper
import venusian


@view_config(route_name='admin_site', context=_rsr.AdminSite,
             renderer='pyramid_backend:templates/index.mak')
def admin_site_home_view(request):
    """
    :type request: pyramid.request.Request
    """
    models = model_helper.get_registered_models()
    return {
        'models': [dict(
            name=m.__name__,
            url=model_helper.get_model_url(request, m)
        ) for m in models]
    }


def cell_datatype(val):
    if val is None:
        return 'none'
    if isinstance(val, bool):
        return 'bool'
    if isinstance(val, (int, long, float)):
        return 'number'
    if isinstance(val, datetime):
        return 'datetime'
    if isinstance(val, basestring):
        return 'longtext' if len(val) > 40 else 'text'
    if inspect.isclass(type(val)):
        return 'class'
    return 'generic'


from . import model_view
from .model_view import ModelView


def add_model_view(config, model, view_cls=None, actions=None):
    """
    :type config: pyramid.config.Configurator
    """
    model_helper.register_model(model)

    if view_cls is None:
        view_cls = ModelView
    mgr = model.__backend_manager__
    """:type : pyramid_backend.backend_manager.Manager"""
    if actions is None:
        actions = mgr.default_actions
    for action_conf in actions:
        action_name, action_conf = mgr.normalize_action(action_conf)
        if 'view' not in action_conf:
            action_conf['view'] = view_cls
        _add_action_view(config, model, action_name, **action_conf)


def _add_action_view(config, model, action_name, default_context=None, **kwargs):
    model_helper.register_model(model)
    mgr = model.__backend_manager__
    """:type : pyramid_backend.backend_manager.Manager"""
    if 'context' not in kwargs:
        kwargs['context'] = default_context
    if 'route_name' not in kwargs:
        kwargs['route_name'] = 'admin_site'
    if 'name' not in kwargs:
        kwargs['name'] = action_name
    action_name, action_conf = mgr.add_action((action_name, kwargs))
    view_conf = {k: v for k, v in action_conf.items() if not k.startswith('_')}
    config.add_view(**view_conf)


def add_model_action(config, model, action_name, **kwargs):
    model_helper.register_model(model)
    mgr = model.__backend_manager__
    _add_action_view(config, model, action_name, mgr.ModelResource, **kwargs)


def add_object_action(config, model, action_name, **kwargs):
    model_helper.register_model(model)
    mgr = model.__backend_manager__
    _add_action_view(config, model, action_name, mgr.ObjectResource, **kwargs)


class model_view_config(object):
    def __init__(self, model, actions=None):
        assert model, "You have to specify model"
        self.model = model
        self.actions = actions

    def view_config(self, scanner, name, ob):
        add_model_view(scanner.config,
                       self.model,
                       ob,
                       actions=self.actions)

    def __call__(self, wrapped):
        assert inspect.isclass(wrapped), \
            '@model_view_config can decorate to class only (decorated to %s)' % type(wrapped)
        info = venusian.attach(wrapped, self.view_config)
        return wrapped


class _action_view_config(object):
    _add_action_fn = None

    def __init__(self, model, name=None, **kwargs):
        assert model, "You have to specify model"
        self.model = model
        self.action_name = name
        self.conf = kwargs

    def __call__(self, wrapped):
        def callback(scanner, name, ob):
            conf = self.conf.copy()
            conf['view'] = ob
            self.__class__._add_action_fn(scanner.config,
                             self.model,
                             self.action_name,
                             **conf)
        info = venusian.attach(wrapped, callback)
        if info.scope == 'class':
            # if the decorator was attached to a method in a class, or
            # otherwise executed at class scope, we need to set an
            # 'attr' into the settings if one isn't already in there
            if not self.conf.get('attr'):
                self.conf['attr'] = wrapped.__name__
        if self.action_name is None:
            self.action_name = wrapped.__name__
        return wrapped


class model_action_config(_action_view_config):
    _add_action_fn = staticmethod(add_model_action)


class object_action_config(_action_view_config):
    _add_action_fn = staticmethod(add_object_action)



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()

requires = [
    'pyramid >= 1.5a1',
    'deform',
    'pyramid_layout',
    'webhelpers',
    ]

setup(name='pyramid_backend',
      version='1.0.5',
      description='pyramid_backend',
      long_description=README,
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        ],
      author='Hoc .T Do',
      author_email='hoc3010@gmail.com',
      url='',
      keywords='web wsgi bfg pylons pyramid',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      test_suite='notification',
      install_requires=requires,
      )